import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'authentication.m.dart';

import 'liveConsultation.m.dart';


class AuthenticationService extends JsonRpcClientBase {
	AuthenticationService(
		String host, {
		String serviceName = "IAuthenticationService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => TokenDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => ValidateTokenResult.fromJson(map));
		FJsonConvert.setDecoder((map) => PageResult<TokenDTO>.fromJson(map));
	}

	Future<String> getAuthorizationAsync(AuthenticationRequest request) async {
		var rpcRst = await call("GetAuthorizationAsync", request);
		return rpcRst;
	}

	Future<String> getVodAuthorizationAsync(TokenRequest request) async {
		var rpcRst = await call("GetVodAuthorizationAsync", request);
		return rpcRst;
	}

	Future<String> getServerDefaultTokenAsync() async {
		var rpcRst = await call("GetServerDefaultTokenAsync", );
		return rpcRst;
	}

	Future<TokenDTO> applyTokenAsync(ApplyTokenRequest request) async {
		var rpcRst = await call("ApplyTokenAsync", request);
		var result = TokenDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<TokenDTO> getTokenAsync(TokenRequest request) async {
		var rpcRst = await call("GetTokenAsync", request);
		var result = TokenDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<ValidateTokenResult> validateTokenAsync(ValidateTokenRequest request) async {
		var rpcRst = await call("ValidateTokenAsync", request);
		var result = ValidateTokenResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<List<TokenDTO>> getTokensWithClientIdAsync(GetTokensWithClientIdRequest request) async {
		var rpcRst = await call("GetTokensWithClientIdAsync", request);
		var result = (rpcRst as List).map((e)=>TokenDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<List<TokenDTO>> getTokenWithClientIdsAsync(GetTokenWithClientIdsRequest request) async {
		var rpcRst = await call("GetTokenWithClientIdsAsync", request);
		var result = (rpcRst as List).map((e)=>TokenDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<List<TokenDTO>> getTokenWithValuesAsync(GetTokenWithValuesRequest request) async {
		var rpcRst = await call("GetTokenWithValuesAsync", request);
		var result = (rpcRst as List).map((e)=>TokenDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<bool> setOnlineStateAsync(SetOnlineStateRequest request) async {
		var rpcRst = await call("SetOnlineStateAsync", request);
		return rpcRst;
	}

	Future<bool> setIPAddressAsync(SetIPAddressRequest request) async {
		var rpcRst = await call("SetIPAddressAsync", request);
		return rpcRst;
	}

	Future<bool> logOffAsync(TokenRequest request) async {
		var rpcRst = await call("LogOffAsync", request);
		return rpcRst;
	}

	Future<PageResult<TokenDTO>> getPagedTokensAsync(GetPagedTokensOnlineRequest request) async {
		var rpcRst = await call("GetPagedTokensAsync", request);
		var result = PageResult<TokenDTO>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> setTokenInstallVersionAsync(SetTokenInstallVersionRequest request) async {
		var rpcRst = await call("SetTokenInstallVersionAsync", request);
		return rpcRst;
	}

}

