import 'liveConsultation.m.dart';
import 'notification.m.dart';
import 'connect.m.dart';

import 'package:fis_jsonrpc/utils.dart';

import 'package:fis_common/json_convert.dart';

class UploadDeviceDTO extends BaseDTO{
	String? serialNumber;
	String? deviceModel;
	String? deviceType;
	String? deviceSoftwareVersion;
	String? sDKSoftwareVersion;

	UploadDeviceDTO({
		this.serialNumber,
		this.deviceModel,
		this.deviceType,
		this.deviceSoftwareVersion,
		this.sDKSoftwareVersion,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory UploadDeviceDTO.fromJson(Map<String, dynamic> map) {
		return UploadDeviceDTO( 
			serialNumber: map['SerialNumber'],
			deviceModel: map['DeviceModel'],
			deviceType: map['DeviceType'],
			deviceSoftwareVersion: map['DeviceSoftwareVersion'],
			sDKSoftwareVersion: map['SDKSoftwareVersion'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(serialNumber != null)
			map['SerialNumber'] = serialNumber;
		if(deviceModel != null)
			map['DeviceModel'] = deviceModel;
		if(deviceType != null)
			map['DeviceType'] = deviceType;
		if(deviceSoftwareVersion != null)
			map['DeviceSoftwareVersion'] = deviceSoftwareVersion;
		if(sDKSoftwareVersion != null)
			map['SDKSoftwareVersion'] = sDKSoftwareVersion;
		return map;
	}
}

class CreateDeviceRequest extends UploadDeviceDTO{

	CreateDeviceRequest({
		String? serialNumber,
		String? deviceModel,
		String? deviceType,
		String? deviceSoftwareVersion,
		String? sDKSoftwareVersion,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			serialNumber: serialNumber,
			deviceModel: deviceModel,
			deviceType: deviceType,
			deviceSoftwareVersion: deviceSoftwareVersion,
			sDKSoftwareVersion: sDKSoftwareVersion,
			createTime: createTime,
			updateTime: updateTime,
		);

	factory CreateDeviceRequest.fromJson(Map<String, dynamic> map) {
		return CreateDeviceRequest( 
			serialNumber: map['SerialNumber'],
			deviceModel: map['DeviceModel'],
			deviceType: map['DeviceType'],
			deviceSoftwareVersion: map['DeviceSoftwareVersion'],
			sDKSoftwareVersion: map['SDKSoftwareVersion'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}

class DictionaryLanguageConfigDTO {
	String? language;
	String? value;

	DictionaryLanguageConfigDTO({
		this.language,
		this.value,
	});

	factory DictionaryLanguageConfigDTO.fromJson(Map<String, dynamic> map) {
		return DictionaryLanguageConfigDTO( 
			language: map['Language'],
			value: map['Value'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(language != null)
			map['Language'] = language;
		if(value != null)
			map['Value'] = value;
		return map;
	}
}

class DeviceExtendInfoDTO extends DeviceInfoDTO{
	String? organizationName;
	String? departmentName;
	bool isOnline;
	bool isLiving;
	String? organizationDirectorCode;
	String? organizationDirectorUserName;
	String? organizationDirectorFullName;
	List<DictionaryLanguageConfigDTO >? languageConfigs;
	bool isEmergencyDevice;

	DeviceExtendInfoDTO({
		this.organizationName,
		this.departmentName,
		this.isOnline = false,
		this.isLiving = false,
		this.organizationDirectorCode,
		this.organizationDirectorUserName,
		this.organizationDirectorFullName,
		this.languageConfigs,
		this.isEmergencyDevice = false,
		String? deviceCode,
		String? serialNumber,
		String? password,
		String? name,
		String? description,
		String? deviceModel,
		String? deviceType,
		String? headPicUrl,
		String? deviceSoftwareVersion,
		String? sDKSoftwareVersion,
		String? organizationCode,
		String? departmentCode,
		String? shortCode,
		bool isAutoShared = false,
		bool isEncryptedShow = false,
		DateTime? lastLoginTime,
		String? systemVersion,
		String? cPUModel,
		String? systemLanguage,
		List<String >? diagnosisModules,
		List<String >? reportPosterCodes,
		bool mergedChannel = false,
		int mergedVideoOutputWidth = 0,
		int mergedVideoOutputHeight = 0,
		List<VideoDeviceDTO >? videoDeviceInfos,
		DownloadModeSettingEnum downloadModeSetting = DownloadModeSettingEnum.Auto,
		bool liveOpened = false,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			deviceCode: deviceCode,
			serialNumber: serialNumber,
			password: password,
			name: name,
			description: description,
			deviceModel: deviceModel,
			deviceType: deviceType,
			headPicUrl: headPicUrl,
			deviceSoftwareVersion: deviceSoftwareVersion,
			sDKSoftwareVersion: sDKSoftwareVersion,
			organizationCode: organizationCode,
			departmentCode: departmentCode,
			shortCode: shortCode,
			isAutoShared: isAutoShared,
			isEncryptedShow: isEncryptedShow,
			lastLoginTime: lastLoginTime,
			systemVersion: systemVersion,
			cPUModel: cPUModel,
			systemLanguage: systemLanguage,
			diagnosisModules: diagnosisModules,
			reportPosterCodes: reportPosterCodes,
			mergedChannel: mergedChannel,
			mergedVideoOutputWidth: mergedVideoOutputWidth,
			mergedVideoOutputHeight: mergedVideoOutputHeight,
			videoDeviceInfos: videoDeviceInfos,
			downloadModeSetting: downloadModeSetting,
			liveOpened: liveOpened,
			createTime: createTime,
			updateTime: updateTime,
		);

	factory DeviceExtendInfoDTO.fromJson(Map<String, dynamic> map) {
		return DeviceExtendInfoDTO( 
			organizationName: map['OrganizationName'],
			departmentName: map['DepartmentName'],
			isOnline: map['IsOnline'],
			isLiving: map['IsLiving'],
			organizationDirectorCode: map['OrganizationDirectorCode'],
			organizationDirectorUserName: map['OrganizationDirectorUserName'],
			organizationDirectorFullName: map['OrganizationDirectorFullName'],
			languageConfigs: map['LanguageConfigs'] != null ? (map['LanguageConfigs'] as List).map((e)=>DictionaryLanguageConfigDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			isEmergencyDevice: map['IsEmergencyDevice'],
			deviceCode: map['DeviceCode'],
			serialNumber: map['SerialNumber'],
			password: map['Password'],
			name: map['Name'],
			description: map['Description'],
			deviceModel: map['DeviceModel'],
			deviceType: map['DeviceType'],
			headPicUrl: map['HeadPicUrl'],
			deviceSoftwareVersion: map['DeviceSoftwareVersion'],
			sDKSoftwareVersion: map['SDKSoftwareVersion'],
			organizationCode: map['OrganizationCode'],
			departmentCode: map['DepartmentCode'],
			shortCode: map['ShortCode'],
			isAutoShared: map['IsAutoShared'],
			isEncryptedShow: map['IsEncryptedShow'],
			lastLoginTime: map['LastLoginTime'] != null ? DateTime.parse(map['LastLoginTime']) : null,
			systemVersion: map['SystemVersion'],
			cPUModel: map['CPUModel'],
			systemLanguage: map['SystemLanguage'],
			diagnosisModules: map['DiagnosisModules'] != null ? map['DiagnosisModules'].cast<String>().toList() : null,
			reportPosterCodes: map['ReportPosterCodes'] != null ? map['ReportPosterCodes'].cast<String>().toList() : null,
			mergedChannel: map['MergedChannel'],
			mergedVideoOutputWidth: map['MergedVideoOutputWidth'],
			mergedVideoOutputHeight: map['MergedVideoOutputHeight'],
			videoDeviceInfos: map['VideoDeviceInfos'] != null ? (map['VideoDeviceInfos'] as List).map((e)=>VideoDeviceDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			downloadModeSetting: DownloadModeSettingEnum.values.firstWhere((e) => e.index == map['DownloadModeSetting']),
			liveOpened: map['LiveOpened'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationName != null)
			map['OrganizationName'] = organizationName;
		if(departmentName != null)
			map['DepartmentName'] = departmentName;
		map['IsOnline'] = isOnline;
		map['IsLiving'] = isLiving;
		if(organizationDirectorCode != null)
			map['OrganizationDirectorCode'] = organizationDirectorCode;
		if(organizationDirectorUserName != null)
			map['OrganizationDirectorUserName'] = organizationDirectorUserName;
		if(organizationDirectorFullName != null)
			map['OrganizationDirectorFullName'] = organizationDirectorFullName;
		if(languageConfigs != null)
			map['LanguageConfigs'] = languageConfigs;
		map['IsEmergencyDevice'] = isEmergencyDevice;
		return map;
	}
}

class GetDeviceRequest extends TokenRequest{
	String? deviceCode;

	GetDeviceRequest({
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetDeviceRequest.fromJson(Map<String, dynamic> map) {
		return GetDeviceRequest( 
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class GetDeviceByShortCodeRequest extends TokenRequest{
	String? shortCode;

	GetDeviceByShortCodeRequest({
		this.shortCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetDeviceByShortCodeRequest.fromJson(Map<String, dynamic> map) {
		return GetDeviceByShortCodeRequest( 
			shortCode: map['ShortCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(shortCode != null)
			map['ShortCode'] = shortCode;
		return map;
	}
}

class PageCollection<T> {
	int currentPage;
	int pageSize;
	int dataCount;
	List<T>? pageData;

	PageCollection({
		this.currentPage = 0,
		this.pageSize = 0,
		this.dataCount = 0,
		this.pageData,
	});

	factory PageCollection.fromJson(Map<String, dynamic> map) {
		List<T> pageDataList = [];
		if (map['PageData'] != null) {
			pageDataList.addAll(
					(map['PageData'] as List).map((e) => FJsonConvert.fromJson<T>(e)!));
		}
		return PageCollection( 
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			dataCount: map['DataCount'],
			pageData: pageDataList,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['CurrentPage'] = currentPage;
		map['PageSize'] = pageSize;
		map['DataCount'] = dataCount;
		if(pageData != null)
			map['PageData'] = pageData;
		return map;
	}
}

class PageFilterRequest extends BaseRequest{
	int currentPage;
	int pageSize;
	Map<String,String>? filter;
	bool isFuzzy;

	PageFilterRequest({
		this.currentPage = 0,
		this.pageSize = 0,
		this.filter,
		this.isFuzzy = false,
	}) : super(
		);

	factory PageFilterRequest.fromJson(Map<String, dynamic> map) {
		return PageFilterRequest( 
			currentPage: map['CurrentPage'],
			pageSize: map['PageSize'],
			filter: map['Filter'] != null ? map['Filter'].cast<String,String>() : null,
			isFuzzy: map['IsFuzzy'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['CurrentPage'] = currentPage;
		map['PageSize'] = pageSize;
		if(filter != null)
			map['Filter'] = filter;
		map['IsFuzzy'] = isFuzzy;
		return map;
	}
}

class BindDeviceRequest extends TokenRequest{
	String? serialNumber;
	String? name;
	String? description;
	String? headPicUrl;
	String? organizationCode;
	String? departmentCode;
	String? shortCode;
	bool isAutoShared;

	BindDeviceRequest({
		this.serialNumber,
		this.name,
		this.description,
		this.headPicUrl,
		this.organizationCode,
		this.departmentCode,
		this.shortCode,
		this.isAutoShared = false,
		String? token,
	}) : super(
			token: token,
		);

	factory BindDeviceRequest.fromJson(Map<String, dynamic> map) {
		return BindDeviceRequest( 
			serialNumber: map['SerialNumber'],
			name: map['Name'],
			description: map['Description'],
			headPicUrl: map['HeadPicUrl'],
			organizationCode: map['OrganizationCode'],
			departmentCode: map['DepartmentCode'],
			shortCode: map['ShortCode'],
			isAutoShared: map['IsAutoShared'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(serialNumber != null)
			map['SerialNumber'] = serialNumber;
		if(name != null)
			map['Name'] = name;
		if(description != null)
			map['Description'] = description;
		if(headPicUrl != null)
			map['HeadPicUrl'] = headPicUrl;
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(departmentCode != null)
			map['DepartmentCode'] = departmentCode;
		if(shortCode != null)
			map['ShortCode'] = shortCode;
		map['IsAutoShared'] = isAutoShared;
		return map;
	}
}

class ModifyDeviceRequest extends TokenRequest{
	String? deviceCode;
	String? name;
	String? headPicUrl;
	String? departmentCode;
	bool isAutoShared;

	ModifyDeviceRequest({
		this.deviceCode,
		this.name,
		this.headPicUrl,
		this.departmentCode,
		this.isAutoShared = false,
		String? token,
	}) : super(
			token: token,
		);

	factory ModifyDeviceRequest.fromJson(Map<String, dynamic> map) {
		return ModifyDeviceRequest( 
			deviceCode: map['DeviceCode'],
			name: map['Name'],
			headPicUrl: map['HeadPicUrl'],
			departmentCode: map['DepartmentCode'],
			isAutoShared: map['IsAutoShared'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(name != null)
			map['Name'] = name;
		if(headPicUrl != null)
			map['HeadPicUrl'] = headPicUrl;
		if(departmentCode != null)
			map['DepartmentCode'] = departmentCode;
		map['IsAutoShared'] = isAutoShared;
		return map;
	}
}

enum DictionaryTypeEnum {
	DeviceModel,
	DeviceType,
}

class CreateDictionaryItemRequest extends TokenRequest{
	DictionaryTypeEnum dictionaryType;
	String? dictionaryItemValue;
	String? parentCode;

	CreateDictionaryItemRequest({
		this.dictionaryType = DictionaryTypeEnum.DeviceModel,
		this.dictionaryItemValue,
		this.parentCode,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateDictionaryItemRequest.fromJson(Map<String, dynamic> map) {
		return CreateDictionaryItemRequest( 
			dictionaryType: DictionaryTypeEnum.values.firstWhere((e) => e.index == map['DictionaryType']),
			dictionaryItemValue: map['DictionaryItemValue'],
			parentCode: map['ParentCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['DictionaryType'] = dictionaryType.index;
		if(dictionaryItemValue != null)
			map['DictionaryItemValue'] = dictionaryItemValue;
		if(parentCode != null)
			map['ParentCode'] = parentCode;
		return map;
	}
}

class DictionaryDTO extends BaseDTO{
	String? dictionaryCode;
	DictionaryTypeEnum dictionaryType;
	String? value;
	String? parentCode;
	List<DictionaryLanguageConfigDTO >? languageConfigs;

	DictionaryDTO({
		this.dictionaryCode,
		this.dictionaryType = DictionaryTypeEnum.DeviceModel,
		this.value,
		this.parentCode,
		this.languageConfigs,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory DictionaryDTO.fromJson(Map<String, dynamic> map) {
		return DictionaryDTO( 
			dictionaryCode: map['DictionaryCode'],
			dictionaryType: DictionaryTypeEnum.values.firstWhere((e) => e.index == map['DictionaryType']),
			value: map['Value'],
			parentCode: map['ParentCode'],
			languageConfigs: map['LanguageConfigs'] != null ? (map['LanguageConfigs'] as List).map((e)=>DictionaryLanguageConfigDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(dictionaryCode != null)
			map['DictionaryCode'] = dictionaryCode;
		map['DictionaryType'] = dictionaryType.index;
		if(value != null)
			map['Value'] = value;
		if(parentCode != null)
			map['ParentCode'] = parentCode;
		if(languageConfigs != null)
			map['LanguageConfigs'] = languageConfigs;
		return map;
	}
}

class FindDeviceModelItemsRequest extends TokenRequest{
	DictionaryTypeEnum dictionaryType;
	String? deviceTypeCode;

	FindDeviceModelItemsRequest({
		this.dictionaryType = DictionaryTypeEnum.DeviceModel,
		this.deviceTypeCode,
		String? token,
	}) : super(
			token: token,
		);

	factory FindDeviceModelItemsRequest.fromJson(Map<String, dynamic> map) {
		return FindDeviceModelItemsRequest( 
			dictionaryType: DictionaryTypeEnum.values.firstWhere((e) => e.index == map['DictionaryType']),
			deviceTypeCode: map['DeviceTypeCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['DictionaryType'] = dictionaryType.index;
		if(deviceTypeCode != null)
			map['DeviceTypeCode'] = deviceTypeCode;
		return map;
	}
}

class FindDeviceTypeItemsRequest extends TokenRequest{
	DictionaryTypeEnum dictionaryType;

	FindDeviceTypeItemsRequest({
		this.dictionaryType = DictionaryTypeEnum.DeviceModel,
		String? token,
	}) : super(
			token: token,
		);

	factory FindDeviceTypeItemsRequest.fromJson(Map<String, dynamic> map) {
		return FindDeviceTypeItemsRequest( 
			dictionaryType: DictionaryTypeEnum.values.firstWhere((e) => e.index == map['DictionaryType']),
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['DictionaryType'] = dictionaryType.index;
		return map;
	}
}

class CreateShareDeviceToUserRequest extends TokenRequest{
	List<String >? userCodes;
	String? deviceCode;

	CreateShareDeviceToUserRequest({
		this.userCodes,
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateShareDeviceToUserRequest.fromJson(Map<String, dynamic> map) {
		return CreateShareDeviceToUserRequest( 
			userCodes: map['UserCodes'] != null ? map['UserCodes'].cast<String>().toList() : null,
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(userCodes != null)
			map['UserCodes'] = userCodes;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class DeleteShareDeviceToUserRequest extends TokenRequest{
	List<String >? userCodes;
	String? deviceCode;

	DeleteShareDeviceToUserRequest({
		this.userCodes,
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory DeleteShareDeviceToUserRequest.fromJson(Map<String, dynamic> map) {
		return DeleteShareDeviceToUserRequest( 
			userCodes: map['UserCodes'] != null ? map['UserCodes'].cast<String>().toList() : null,
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(userCodes != null)
			map['UserCodes'] = userCodes;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class RemoveDeviceRelevancyRequest extends TokenRequest{
	String? organizationCode;
	String? departmentCode;
	String? deviceCode;

	RemoveDeviceRelevancyRequest({
		this.organizationCode,
		this.departmentCode,
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory RemoveDeviceRelevancyRequest.fromJson(Map<String, dynamic> map) {
		return RemoveDeviceRelevancyRequest( 
			organizationCode: map['OrganizationCode'],
			departmentCode: map['DepartmentCode'],
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		if(departmentCode != null)
			map['DepartmentCode'] = departmentCode;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class GetPersonDeviceRequest extends PageRequest{
	String? keyWord;
	String? deviceType;
	String? deviceModel;
	bool? isOnline;

	GetPersonDeviceRequest({
		this.keyWord,
		this.deviceType,
		this.deviceModel,
		this.isOnline,
		int pageIndex = 0,
		int pageSize = 0,
		String? token,
	}) : super(
			pageIndex: pageIndex,
			pageSize: pageSize,
			token: token,
		);

	factory GetPersonDeviceRequest.fromJson(Map<String, dynamic> map) {
		return GetPersonDeviceRequest( 
			keyWord: map['KeyWord'],
			deviceType: map['DeviceType'],
			deviceModel: map['DeviceModel'],
			isOnline: map['IsOnline'],
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(keyWord != null)
			map['KeyWord'] = keyWord;
		if(deviceType != null)
			map['DeviceType'] = deviceType;
		if(deviceModel != null)
			map['DeviceModel'] = deviceModel;
		if(isOnline != null)
			map['IsOnline'] = isOnline;
		return map;
	}
}

class FindDevicesByOrganizationCodeRequest extends TokenRequest{
	String? organizationCode;

	FindDevicesByOrganizationCodeRequest({
		this.organizationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory FindDevicesByOrganizationCodeRequest.fromJson(Map<String, dynamic> map) {
		return FindDevicesByOrganizationCodeRequest( 
			organizationCode: map['OrganizationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(organizationCode != null)
			map['OrganizationCode'] = organizationCode;
		return map;
	}
}

class SelectItemDTO {
	String? key;
	String? value;

	SelectItemDTO({
		this.key,
		this.value,
	});

	factory SelectItemDTO.fromJson(Map<String, dynamic> map) {
		return SelectItemDTO( 
			key: map['Key'],
			value: map['Value'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(key != null)
			map['Key'] = key;
		if(value != null)
			map['Value'] = value;
		return map;
	}
}

class DeviceServerSettingResult {
	Map<String,String>? serverConfigList;
	bool isUploadThumbnail;
	OrganizationPatientTypeEnum patientType;
	int heartRateSeconds;
	String? notificationUrl;
	bool mergedChannel;
	int liveConsultationRateSeconds;
	int mergedVideoOutputWidth;
	int mergedVideoOutputHeight;
	bool isSelfRtcService;
	int remoteControlAskTimeoutSec;

	DeviceServerSettingResult({
		this.serverConfigList,
		this.isUploadThumbnail = false,
		this.patientType = OrganizationPatientTypeEnum.Person,
		this.heartRateSeconds = 0,
		this.notificationUrl,
		this.mergedChannel = false,
		this.liveConsultationRateSeconds = 0,
		this.mergedVideoOutputWidth = 0,
		this.mergedVideoOutputHeight = 0,
		this.isSelfRtcService = false,
		this.remoteControlAskTimeoutSec = 0,
	});

	factory DeviceServerSettingResult.fromJson(Map<String, dynamic> map) {
		return DeviceServerSettingResult( 
			serverConfigList: map['ServerConfigList'] != null ? map['ServerConfigList'].cast<String,String>() : null,
			isUploadThumbnail: map['IsUploadThumbnail'],
			patientType: OrganizationPatientTypeEnum.values.firstWhere((e) => e.index == map['PatientType']),
			heartRateSeconds: map['HeartRateSeconds'],
			notificationUrl: map['NotificationUrl'],
			mergedChannel: map['MergedChannel'],
			liveConsultationRateSeconds: map['LiveConsultationRateSeconds'],
			mergedVideoOutputWidth: map['MergedVideoOutputWidth'],
			mergedVideoOutputHeight: map['MergedVideoOutputHeight'],
			isSelfRtcService: map['IsSelfRtcService'],
			remoteControlAskTimeoutSec: map['RemoteControlAskTimeoutSec'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(serverConfigList != null)
			map['ServerConfigList'] = serverConfigList;
		map['IsUploadThumbnail'] = isUploadThumbnail;
		map['PatientType'] = patientType.index;
		map['HeartRateSeconds'] = heartRateSeconds;
		if(notificationUrl != null)
			map['NotificationUrl'] = notificationUrl;
		map['MergedChannel'] = mergedChannel;
		map['LiveConsultationRateSeconds'] = liveConsultationRateSeconds;
		map['MergedVideoOutputWidth'] = mergedVideoOutputWidth;
		map['MergedVideoOutputHeight'] = mergedVideoOutputHeight;
		map['IsSelfRtcService'] = isSelfRtcService;
		map['RemoteControlAskTimeoutSec'] = remoteControlAskTimeoutSec;
		return map;
	}
}

class AddDeviceToOrgRequest extends TokenRequest{
	String? uniqueCode;

	AddDeviceToOrgRequest({
		this.uniqueCode,
		String? token,
	}) : super(
			token: token,
		);

	factory AddDeviceToOrgRequest.fromJson(Map<String, dynamic> map) {
		return AddDeviceToOrgRequest( 
			uniqueCode: map['UniqueCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(uniqueCode != null)
			map['UniqueCode'] = uniqueCode;
		return map;
	}
}

class DiagnosisModuleDTO extends BaseDTO{
	String? diagnosisModuleCode;
	String? diagnosisModule;
	bool enabled;

	DiagnosisModuleDTO({
		this.diagnosisModuleCode,
		this.diagnosisModule,
		this.enabled = false,
		DateTime? createTime,
		DateTime? updateTime,
	}) : super(
			createTime: createTime,
			updateTime: updateTime,
		);

	factory DiagnosisModuleDTO.fromJson(Map<String, dynamic> map) {
		return DiagnosisModuleDTO( 
			diagnosisModuleCode: map['DiagnosisModuleCode'],
			diagnosisModule: map['DiagnosisModule'],
			enabled: map['Enabled'],
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			updateTime: map['UpdateTime'] != null ? DateTime.parse(map['UpdateTime']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(diagnosisModuleCode != null)
			map['DiagnosisModuleCode'] = diagnosisModuleCode;
		if(diagnosisModule != null)
			map['DiagnosisModule'] = diagnosisModule;
		map['Enabled'] = enabled;
		return map;
	}
}

class FindDeviceDiagnosisModulesRequest extends TokenRequest{
	String? deviceCode;

	FindDeviceDiagnosisModulesRequest({
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory FindDeviceDiagnosisModulesRequest.fromJson(Map<String, dynamic> map) {
		return FindDeviceDiagnosisModulesRequest( 
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class ModifyDeviceDiagnosisModuleStateRequest extends TokenRequest{
	String? deviceCode;
	String? diagnosisModule;
	bool enabled;

	ModifyDeviceDiagnosisModuleStateRequest({
		this.deviceCode,
		this.diagnosisModule,
		this.enabled = false,
		String? token,
	}) : super(
			token: token,
		);

	factory ModifyDeviceDiagnosisModuleStateRequest.fromJson(Map<String, dynamic> map) {
		return ModifyDeviceDiagnosisModuleStateRequest( 
			deviceCode: map['DeviceCode'],
			diagnosisModule: map['DiagnosisModule'],
			enabled: map['Enabled'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(diagnosisModule != null)
			map['DiagnosisModule'] = diagnosisModule;
		map['Enabled'] = enabled;
		return map;
	}
}

class ReportVideoDeviceInfoResult {
	bool success;
	List<VideoDeviceDTO >? videoDeviceOutputInfos;

	ReportVideoDeviceInfoResult({
		this.success = false,
		this.videoDeviceOutputInfos,
	});

	factory ReportVideoDeviceInfoResult.fromJson(Map<String, dynamic> map) {
		return ReportVideoDeviceInfoResult( 
			success: map['Success'],
			videoDeviceOutputInfos: map['VideoDeviceOutputInfos'] != null ? (map['VideoDeviceOutputInfos'] as List).map((e)=>VideoDeviceDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Success'] = success;
		if(videoDeviceOutputInfos != null)
			map['VideoDeviceOutputInfos'] = videoDeviceOutputInfos;
		return map;
	}
}

class VideoDeviceInfo {
	String? videoDeviceId;
	VideoDeviceSourceTypeEnum videoDeviceSourceType;
	int width;
	int height;
	int videoFps;
	int videoBitrate;
	int minVideoBitrate;

	VideoDeviceInfo({
		this.videoDeviceId,
		this.videoDeviceSourceType = VideoDeviceSourceTypeEnum.Desktop,
		this.width = 0,
		this.height = 0,
		this.videoFps = 0,
		this.videoBitrate = 0,
		this.minVideoBitrate = 0,
	});

	factory VideoDeviceInfo.fromJson(Map<String, dynamic> map) {
		return VideoDeviceInfo( 
			videoDeviceId: map['VideoDeviceId'],
			videoDeviceSourceType: VideoDeviceSourceTypeEnum.values.firstWhere((e) => e.index == map['VideoDeviceSourceType']),
			width: map['Width'],
			height: map['Height'],
			videoFps: map['VideoFps'],
			videoBitrate: map['VideoBitrate'],
			minVideoBitrate: map['MinVideoBitrate'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(videoDeviceId != null)
			map['VideoDeviceId'] = videoDeviceId;
		map['VideoDeviceSourceType'] = videoDeviceSourceType.index;
		map['Width'] = width;
		map['Height'] = height;
		map['VideoFps'] = videoFps;
		map['VideoBitrate'] = videoBitrate;
		map['MinVideoBitrate'] = minVideoBitrate;
		return map;
	}
}

class ReportVideoDeviceInfoRequest extends TokenRequest{
	bool liveOpened;
	List<VideoDeviceInfo >? videoDeviceInfos;

	ReportVideoDeviceInfoRequest({
		this.liveOpened = false,
		this.videoDeviceInfos,
		String? token,
	}) : super(
			token: token,
		);

	factory ReportVideoDeviceInfoRequest.fromJson(Map<String, dynamic> map) {
		return ReportVideoDeviceInfoRequest( 
			liveOpened: map['LiveOpened'],
			videoDeviceInfos: map['VideoDeviceInfos'] != null ? (map['VideoDeviceInfos'] as List).map((e)=>VideoDeviceInfo.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['LiveOpened'] = liveOpened;
		if(videoDeviceInfos != null)
			map['VideoDeviceInfos'] = videoDeviceInfos;
		return map;
	}
}

class ApplicationSettingInfoDTO {
	String? id;
	String? name;
	bool isPreferred;
	bool isUserDefined;
	bool isHidden;

	ApplicationSettingInfoDTO({
		this.id,
		this.name,
		this.isPreferred = false,
		this.isUserDefined = false,
		this.isHidden = false,
	});

	factory ApplicationSettingInfoDTO.fromJson(Map<String, dynamic> map) {
		return ApplicationSettingInfoDTO( 
			id: map['Id'],
			name: map['Name'],
			isPreferred: map['IsPreferred'],
			isUserDefined: map['IsUserDefined'],
			isHidden: map['IsHidden'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(id != null)
			map['Id'] = id;
		if(name != null)
			map['Name'] = name;
		map['IsPreferred'] = isPreferred;
		map['IsUserDefined'] = isUserDefined;
		map['IsHidden'] = isHidden;
		return map;
	}
}

class ProbeInfoDTO {
	String? name;
	List<String >? applications;
	List<ApplicationSettingInfoDTO >? applicationInfos;

	ProbeInfoDTO({
		this.name,
		this.applications,
		this.applicationInfos,
	});

	factory ProbeInfoDTO.fromJson(Map<String, dynamic> map) {
		return ProbeInfoDTO( 
			name: map['Name'],
			applications: map['Applications'] != null ? map['Applications'].cast<String>().toList() : null,
			applicationInfos: map['ApplicationInfos'] != null ? (map['ApplicationInfos'] as List).map((e)=>ApplicationSettingInfoDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(name != null)
			map['Name'] = name;
		if(applications != null)
			map['Applications'] = applications;
		if(applicationInfos != null)
			map['ApplicationInfos'] = applicationInfos;
		return map;
	}
}

class ProbeApplicationInfoDTO {
	List<ProbeInfoDTO >? probes;
	String? activeProbe;
	String? activeApplication;

	ProbeApplicationInfoDTO({
		this.probes,
		this.activeProbe,
		this.activeApplication,
	});

	factory ProbeApplicationInfoDTO.fromJson(Map<String, dynamic> map) {
		return ProbeApplicationInfoDTO( 
			probes: map['Probes'] != null ? (map['Probes'] as List).map((e)=>ProbeInfoDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			activeProbe: map['ActiveProbe'],
			activeApplication: map['ActiveApplication'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(probes != null)
			map['Probes'] = probes;
		if(activeProbe != null)
			map['ActiveProbe'] = activeProbe;
		if(activeApplication != null)
			map['ActiveApplication'] = activeApplication;
		return map;
	}
}

class ControlParameterDTO {
	bool canExecute;
	List<ControlParameterDTO >? children;
	String? description;
	String? displayValue;
	String? parentDescription;
	String? valuesMapString;

	ControlParameterDTO({
		this.canExecute = false,
		this.children,
		this.description,
		this.displayValue,
		this.parentDescription,
		this.valuesMapString,
	});

	factory ControlParameterDTO.fromJson(Map<String, dynamic> map) {
		return ControlParameterDTO( 
			canExecute: map['CanExecute'],
			children: map['Children'] != null ? (map['Children'] as List).map((e)=>ControlParameterDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			description: map['Description'],
			displayValue: map['DisplayValue'],
			parentDescription: map['ParentDescription'],
			valuesMapString: map['ValuesMapString'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['CanExecute'] = canExecute;
		if(children != null)
			map['Children'] = children;
		if(description != null)
			map['Description'] = description;
		if(displayValue != null)
			map['DisplayValue'] = displayValue;
		if(parentDescription != null)
			map['ParentDescription'] = parentDescription;
		if(valuesMapString != null)
			map['ValuesMapString'] = valuesMapString;
		return map;
	}
}

class SendControlParameterByDeviceRequest extends TokenRequest{
	String? controlUserCode;
	ProbeApplicationInfoDTO? probeApplication;
	ControlParameterDTO? parameter;

	SendControlParameterByDeviceRequest({
		this.controlUserCode,
		this.probeApplication,
		this.parameter,
		String? token,
	}) : super(
			token: token,
		);

	factory SendControlParameterByDeviceRequest.fromJson(Map<String, dynamic> map) {
		return SendControlParameterByDeviceRequest( 
			controlUserCode: map['ControlUserCode'],
			probeApplication: map['ProbeApplication'] != null ? ProbeApplicationInfoDTO.fromJson(map['ProbeApplication']) : null,
			parameter: map['Parameter'] != null ? ControlParameterDTO.fromJson(map['Parameter']) : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(controlUserCode != null)
			map['ControlUserCode'] = controlUserCode;
		if(probeApplication != null)
			map['ProbeApplication'] = probeApplication;
		if(parameter != null)
			map['Parameter'] = parameter;
		return map;
	}
}

class LiveDataDTO {
	int width;
	int height;
	String? rtmpPushUrl;
	String? rtmpPullUrl;
	String? httpPullUrl;
	String? hlsPullUrl;

	LiveDataDTO({
		this.width = 0,
		this.height = 0,
		this.rtmpPushUrl,
		this.rtmpPullUrl,
		this.httpPullUrl,
		this.hlsPullUrl,
	});

	factory LiveDataDTO.fromJson(Map<String, dynamic> map) {
		return LiveDataDTO( 
			width: map['Width'],
			height: map['Height'],
			rtmpPushUrl: map['RtmpPushUrl'],
			rtmpPullUrl: map['RtmpPullUrl'],
			httpPullUrl: map['HttpPullUrl'],
			hlsPullUrl: map['HlsPullUrl'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['Width'] = width;
		map['Height'] = height;
		if(rtmpPushUrl != null)
			map['RtmpPushUrl'] = rtmpPushUrl;
		if(rtmpPullUrl != null)
			map['RtmpPullUrl'] = rtmpPullUrl;
		if(httpPullUrl != null)
			map['HttpPullUrl'] = httpPullUrl;
		if(hlsPullUrl != null)
			map['HlsPullUrl'] = hlsPullUrl;
		return map;
	}
}

class VideoDeviceInfoDTO {
	String? videoDeviceId;
	VideoDeviceSourceTypeEnum videoDeviceSourceType;
	LiveDataDTO? liveData;

	VideoDeviceInfoDTO({
		this.videoDeviceId,
		this.videoDeviceSourceType = VideoDeviceSourceTypeEnum.Desktop,
		this.liveData,
	});

	factory VideoDeviceInfoDTO.fromJson(Map<String, dynamic> map) {
		return VideoDeviceInfoDTO( 
			videoDeviceId: map['VideoDeviceId'],
			videoDeviceSourceType: VideoDeviceSourceTypeEnum.values.firstWhere((e) => e.index == map['VideoDeviceSourceType']),
			liveData: map['LiveData'] != null ? LiveDataDTO.fromJson(map['LiveData']) : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(videoDeviceId != null)
			map['VideoDeviceId'] = videoDeviceId;
		map['VideoDeviceSourceType'] = videoDeviceSourceType.index;
		if(liveData != null)
			map['LiveData'] = liveData;
		return map;
	}
}

class JoinDeviceLiveRoomResult extends TokenRequest{
	int roomNo;
	String? deviceCode;
	bool mergedChannel;
	int mergedVideoOutputWidth;
	int mergedVideoOutputHeight;
	List<VideoDeviceInfoDTO >? videoDeviceInfos;
	int reportStateIntervalSeconds;

	JoinDeviceLiveRoomResult({
		this.roomNo = 0,
		this.deviceCode,
		this.mergedChannel = false,
		this.mergedVideoOutputWidth = 0,
		this.mergedVideoOutputHeight = 0,
		this.videoDeviceInfos,
		this.reportStateIntervalSeconds = 0,
		String? token,
	}) : super(
			token: token,
		);

	factory JoinDeviceLiveRoomResult.fromJson(Map<String, dynamic> map) {
		return JoinDeviceLiveRoomResult( 
			roomNo: map['RoomNo'],
			deviceCode: map['DeviceCode'],
			mergedChannel: map['MergedChannel'],
			mergedVideoOutputWidth: map['MergedVideoOutputWidth'],
			mergedVideoOutputHeight: map['MergedVideoOutputHeight'],
			videoDeviceInfos: map['VideoDeviceInfos'] != null ? (map['VideoDeviceInfos'] as List).map((e)=>VideoDeviceInfoDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			reportStateIntervalSeconds: map['ReportStateIntervalSeconds'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['RoomNo'] = roomNo;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		map['MergedChannel'] = mergedChannel;
		map['MergedVideoOutputWidth'] = mergedVideoOutputWidth;
		map['MergedVideoOutputHeight'] = mergedVideoOutputHeight;
		if(videoDeviceInfos != null)
			map['VideoDeviceInfos'] = videoDeviceInfos;
		map['ReportStateIntervalSeconds'] = reportStateIntervalSeconds;
		return map;
	}
}

class JoinDeviceLiveRoomRequest extends TokenRequest{
	String? deviceCode;

	JoinDeviceLiveRoomRequest({
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory JoinDeviceLiveRoomRequest.fromJson(Map<String, dynamic> map) {
		return JoinDeviceLiveRoomRequest( 
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class LeaveDeviceLiveRoomRequest extends TokenRequest{
	String? deviceCode;

	LeaveDeviceLiveRoomRequest({
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory LeaveDeviceLiveRoomRequest.fromJson(Map<String, dynamic> map) {
		return LeaveDeviceLiveRoomRequest( 
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class ReportLiveViewStateRequest extends TokenRequest{
	String? deviceCode;

	ReportLiveViewStateRequest({
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory ReportLiveViewStateRequest.fromJson(Map<String, dynamic> map) {
		return ReportLiveViewStateRequest( 
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class CreateLiveShareInfoResult extends TokenRequest{
	String? shareUrl;

	CreateLiveShareInfoResult({
		this.shareUrl,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateLiveShareInfoResult.fromJson(Map<String, dynamic> map) {
		return CreateLiveShareInfoResult( 
			shareUrl: map['ShareUrl'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(shareUrl != null)
			map['ShareUrl'] = shareUrl;
		return map;
	}
}

class CreateLiveShareInfoRequest extends TokenRequest{
	String? deviceCode;

	CreateLiveShareInfoRequest({
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateLiveShareInfoRequest.fromJson(Map<String, dynamic> map) {
		return CreateLiveShareInfoRequest( 
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class JoinDeviceLiveRoomByShareResult {
	int roomNo;
	String? deviceCode;
	bool mergedChannel;
	int mergedVideoOutputWidth;
	int mergedVideoOutputHeight;
	List<VideoDeviceInfoDTO >? videoDeviceInfos;
	int reportStateIntervalSeconds;

	JoinDeviceLiveRoomByShareResult({
		this.roomNo = 0,
		this.deviceCode,
		this.mergedChannel = false,
		this.mergedVideoOutputWidth = 0,
		this.mergedVideoOutputHeight = 0,
		this.videoDeviceInfos,
		this.reportStateIntervalSeconds = 0,
	});

	factory JoinDeviceLiveRoomByShareResult.fromJson(Map<String, dynamic> map) {
		return JoinDeviceLiveRoomByShareResult( 
			roomNo: map['RoomNo'],
			deviceCode: map['DeviceCode'],
			mergedChannel: map['MergedChannel'],
			mergedVideoOutputWidth: map['MergedVideoOutputWidth'],
			mergedVideoOutputHeight: map['MergedVideoOutputHeight'],
			videoDeviceInfos: map['VideoDeviceInfos'] != null ? (map['VideoDeviceInfos'] as List).map((e)=>VideoDeviceInfoDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			reportStateIntervalSeconds: map['ReportStateIntervalSeconds'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['RoomNo'] = roomNo;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		map['MergedChannel'] = mergedChannel;
		map['MergedVideoOutputWidth'] = mergedVideoOutputWidth;
		map['MergedVideoOutputHeight'] = mergedVideoOutputHeight;
		if(videoDeviceInfos != null)
			map['VideoDeviceInfos'] = videoDeviceInfos;
		map['ReportStateIntervalSeconds'] = reportStateIntervalSeconds;
		return map;
	}
}

class JoinDeviceLiveRoomByShareRequest {
	String? shareCode;

	JoinDeviceLiveRoomByShareRequest({
		this.shareCode,
	});

	factory JoinDeviceLiveRoomByShareRequest.fromJson(Map<String, dynamic> map) {
		return JoinDeviceLiveRoomByShareRequest( 
			shareCode: map['ShareCode'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(shareCode != null)
			map['ShareCode'] = shareCode;
		return map;
	}
}

class LeaveDeviceLiveRoomByShareRequest {
	String? deviceCode;
	String? viewerUniqueId;

	LeaveDeviceLiveRoomByShareRequest({
		this.deviceCode,
		this.viewerUniqueId,
	});

	factory LeaveDeviceLiveRoomByShareRequest.fromJson(Map<String, dynamic> map) {
		return LeaveDeviceLiveRoomByShareRequest( 
			deviceCode: map['DeviceCode'],
			viewerUniqueId: map['ViewerUniqueId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(viewerUniqueId != null)
			map['ViewerUniqueId'] = viewerUniqueId;
		return map;
	}
}

class ReportLiveViewStateByShareRequest {
	String? deviceCode;
	String? viewerUniqueId;

	ReportLiveViewStateByShareRequest({
		this.deviceCode,
		this.viewerUniqueId,
	});

	factory ReportLiveViewStateByShareRequest.fromJson(Map<String, dynamic> map) {
		return ReportLiveViewStateByShareRequest( 
			deviceCode: map['DeviceCode'],
			viewerUniqueId: map['ViewerUniqueId'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(viewerUniqueId != null)
			map['ViewerUniqueId'] = viewerUniqueId;
		return map;
	}
}

enum DeviceLiveStateEnum {
	Default,
	Pushing,
	Closed,
	Error,
	Warning,
}

class ReportLiveStateRequest extends TokenRequest{
	int roomNo;
	DeviceLiveStateEnum liveState;
	String? message;

	ReportLiveStateRequest({
		this.roomNo = 0,
		this.liveState = DeviceLiveStateEnum.Default,
		this.message,
		String? token,
	}) : super(
			token: token,
		);

	factory ReportLiveStateRequest.fromJson(Map<String, dynamic> map) {
		return ReportLiveStateRequest( 
			roomNo: map['RoomNo'],
			liveState: DeviceLiveStateEnum.values.firstWhere((e) => e.index == map['LiveState']),
			message: map['Message'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['RoomNo'] = roomNo;
		map['LiveState'] = liveState.index;
		if(message != null)
			map['Message'] = message;
		return map;
	}
}

class CreateLiveRoomInfoResult {
	String? userCode;
	String? userSign;
	int roomNo;
	int appId;
	bool isTrtc;

	CreateLiveRoomInfoResult({
		this.userCode,
		this.userSign,
		this.roomNo = 0,
		this.appId = 0,
		this.isTrtc = false,
	});

	factory CreateLiveRoomInfoResult.fromJson(Map<String, dynamic> map) {
		return CreateLiveRoomInfoResult( 
			userCode: map['UserCode'],
			userSign: map['UserSign'],
			roomNo: map['RoomNo'],
			appId: map['AppId'],
			isTrtc: map['IsTrtc'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(userCode != null)
			map['UserCode'] = userCode;
		if(userSign != null)
			map['UserSign'] = userSign;
		map['RoomNo'] = roomNo;
		map['AppId'] = appId;
		map['IsTrtc'] = isTrtc;
		return map;
	}
}

class CreateLiveRoomInfoRequest {
	String? deviceUniqueCode;
	String? deviceModel;
	String? deviceType;
	String? softwareVersion;

	CreateLiveRoomInfoRequest({
		this.deviceUniqueCode,
		this.deviceModel,
		this.deviceType,
		this.softwareVersion,
	});

	factory CreateLiveRoomInfoRequest.fromJson(Map<String, dynamic> map) {
		return CreateLiveRoomInfoRequest( 
			deviceUniqueCode: map['DeviceUniqueCode'],
			deviceModel: map['DeviceModel'],
			deviceType: map['DeviceType'],
			softwareVersion: map['SoftwareVersion'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(deviceUniqueCode != null)
			map['DeviceUniqueCode'] = deviceUniqueCode;
		if(deviceModel != null)
			map['DeviceModel'] = deviceModel;
		if(deviceType != null)
			map['DeviceType'] = deviceType;
		if(softwareVersion != null)
			map['SoftwareVersion'] = softwareVersion;
		return map;
	}
}

class RemoteControlRequest extends TokenRequest{
	String? controlUserCode;

	RemoteControlRequest({
		this.controlUserCode,
		String? token,
	}) : super(
			token: token,
		);

	factory RemoteControlRequest.fromJson(Map<String, dynamic> map) {
		return RemoteControlRequest( 
			controlUserCode: map['ControlUserCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(controlUserCode != null)
			map['ControlUserCode'] = controlUserCode;
		return map;
	}
}

class UploadConsultationDataRequest extends TokenRequest{
	String? consultationCode;
	String? previewFileToken;
	String? fileToken;
	int fileSize;
	String? coverImageToken;
	String? applicationCategory;
	String? application;
	RemedicalFileDataTypeEnum fileDataType;
	MeasuredResultsDTO? measuredResult;
	ScanImageDTO? commentResult;

	UploadConsultationDataRequest({
		this.consultationCode,
		this.previewFileToken,
		this.fileToken,
		this.fileSize = 0,
		this.coverImageToken,
		this.applicationCategory,
		this.application,
		this.fileDataType = RemedicalFileDataTypeEnum.VinnoVidSingle,
		this.measuredResult,
		this.commentResult,
		String? token,
	}) : super(
			token: token,
		);

	factory UploadConsultationDataRequest.fromJson(Map<String, dynamic> map) {
		return UploadConsultationDataRequest( 
			consultationCode: map['ConsultationCode'],
			previewFileToken: map['PreviewFileToken'],
			fileToken: map['FileToken'],
			fileSize: map['FileSize'],
			coverImageToken: map['CoverImageToken'],
			applicationCategory: map['ApplicationCategory'],
			application: map['Application'],
			fileDataType: RemedicalFileDataTypeEnum.values.firstWhere((e) => e.index == map['FileDataType']),
			measuredResult: map['MeasuredResult'] != null ? MeasuredResultsDTO.fromJson(map['MeasuredResult']) : null,
			commentResult: map['CommentResult'] != null ? ScanImageDTO.fromJson(map['CommentResult']) : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(consultationCode != null)
			map['ConsultationCode'] = consultationCode;
		if(previewFileToken != null)
			map['PreviewFileToken'] = previewFileToken;
		if(fileToken != null)
			map['FileToken'] = fileToken;
		map['FileSize'] = fileSize;
		if(coverImageToken != null)
			map['CoverImageToken'] = coverImageToken;
		if(applicationCategory != null)
			map['ApplicationCategory'] = applicationCategory;
		if(application != null)
			map['Application'] = application;
		map['FileDataType'] = fileDataType.index;
		if(measuredResult != null)
			map['MeasuredResult'] = measuredResult;
		if(commentResult != null)
			map['CommentResult'] = commentResult;
		return map;
	}
}

class DeviceControlParameterDataDTO {
	String? deviceCode;
	String? probeApplication;
	String? parameter;

	DeviceControlParameterDataDTO({
		this.deviceCode,
		this.probeApplication,
		this.parameter,
	});

	factory DeviceControlParameterDataDTO.fromJson(Map<String, dynamic> map) {
		return DeviceControlParameterDataDTO( 
			deviceCode: map['DeviceCode'],
			probeApplication: map['ProbeApplication'],
			parameter: map['Parameter'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(probeApplication != null)
			map['ProbeApplication'] = probeApplication;
		if(parameter != null)
			map['Parameter'] = parameter;
		return map;
	}
}

class GetControlParametersRequest extends TokenRequest{
	String? deviceCode;

	GetControlParametersRequest({
		this.deviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory GetControlParametersRequest.fromJson(Map<String, dynamic> map) {
		return GetControlParametersRequest( 
			deviceCode: map['DeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class ControlDeviceConnectRequest extends BaseControlDeviceRequest{
	String? deviceCode;

	ControlDeviceConnectRequest({
		this.deviceCode,
		ControlDeviceParameterEnum controlType = ControlDeviceParameterEnum.Start,
		String? token,
	}) : super(
			controlType: controlType,
			token: token,
		);

	factory ControlDeviceConnectRequest.fromJson(Map<String, dynamic> map) {
		return ControlDeviceConnectRequest( 
			deviceCode: map['DeviceCode'],
			controlType: ControlDeviceParameterEnum.values.firstWhere((e) => e.index == map['ControlType']),
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class ControlDeviceResponseRequest extends BaseControlDeviceRequest{
	String? userCode;

	ControlDeviceResponseRequest({
		this.userCode,
		ControlDeviceParameterEnum controlType = ControlDeviceParameterEnum.Start,
		String? token,
	}) : super(
			controlType: controlType,
			token: token,
		);

	factory ControlDeviceResponseRequest.fromJson(Map<String, dynamic> map) {
		return ControlDeviceResponseRequest( 
			userCode: map['UserCode'],
			controlType: ControlDeviceParameterEnum.values.firstWhere((e) => e.index == map['ControlType']),
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(userCode != null)
			map['UserCode'] = userCode;
		return map;
	}
}

class ControlDeviceParameterRequest extends BaseControlDeviceParameterRequest{
	String? deviceCode;

	ControlDeviceParameterRequest({
		this.deviceCode,
		List<AdditionParameterDTO >? parameters,
		ControlDeviceParameterEnum controlType = ControlDeviceParameterEnum.Start,
		String? token,
	}) : super(
			parameters: parameters,
			controlType: controlType,
			token: token,
		);

	factory ControlDeviceParameterRequest.fromJson(Map<String, dynamic> map) {
		return ControlDeviceParameterRequest( 
			deviceCode: map['DeviceCode'],
			parameters: map['Parameters'] != null ? (map['Parameters'] as List).map((e)=>AdditionParameterDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			controlType: ControlDeviceParameterEnum.values.firstWhere((e) => e.index == map['ControlType']),
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		return map;
	}
}

class GetRemoteLogRequest extends TokenRequest{
	String? deviceCode;
	DateTime? startTime;
	DateTime? endTime;

	GetRemoteLogRequest({
		this.deviceCode,
		this.startTime,
		this.endTime,
		String? token,
	}) : super(
			token: token,
		);

	factory GetRemoteLogRequest.fromJson(Map<String, dynamic> map) {
		return GetRemoteLogRequest( 
			deviceCode: map['DeviceCode'],
			startTime: map['StartTime'] != null ? DateTime.parse(map['StartTime']) : null,
			endTime: map['EndTime'] != null ? DateTime.parse(map['EndTime']) : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(startTime != null)
			map['StartTime'] = JsonRpcUtils.dateFormat(startTime!);
		if(endTime != null)
			map['EndTime'] = JsonRpcUtils.dateFormat(endTime!);
		return map;
	}
}

class RemoteLogResponseRequest extends TokenRequest{
	String? userCode;
	String? logFileToken;

	RemoteLogResponseRequest({
		this.userCode,
		this.logFileToken,
		String? token,
	}) : super(
			token: token,
		);

	factory RemoteLogResponseRequest.fromJson(Map<String, dynamic> map) {
		return RemoteLogResponseRequest( 
			userCode: map['UserCode'],
			logFileToken: map['LogFileToken'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(userCode != null)
			map['UserCode'] = userCode;
		if(logFileToken != null)
			map['LogFileToken'] = logFileToken;
		return map;
	}
}

class ScanBindDeviceRequest extends TokenRequest{
	String? shortCode;

	ScanBindDeviceRequest({
		this.shortCode,
		String? token,
	}) : super(
			token: token,
		);

	factory ScanBindDeviceRequest.fromJson(Map<String, dynamic> map) {
		return ScanBindDeviceRequest( 
			shortCode: map['ShortCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(shortCode != null)
			map['ShortCode'] = shortCode;
		return map;
	}
}

class ReportBrandModelOutputConfigRequest extends TokenRequest{
	String? brand;
	String? model;
	String? shortCode;
	int videoWidth;
	int videoHeight;

	ReportBrandModelOutputConfigRequest({
		this.brand,
		this.model,
		this.shortCode,
		this.videoWidth = 0,
		this.videoHeight = 0,
		String? token,
	}) : super(
			token: token,
		);

	factory ReportBrandModelOutputConfigRequest.fromJson(Map<String, dynamic> map) {
		return ReportBrandModelOutputConfigRequest( 
			brand: map['Brand'],
			model: map['Model'],
			shortCode: map['ShortCode'],
			videoWidth: map['VideoWidth'],
			videoHeight: map['VideoHeight'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(brand != null)
			map['Brand'] = brand;
		if(model != null)
			map['Model'] = model;
		if(shortCode != null)
			map['ShortCode'] = shortCode;
		map['VideoWidth'] = videoWidth;
		map['VideoHeight'] = videoHeight;
		return map;
	}
}

class BrandModelOutputConfigDTO {
	bool isSelect;
	int videoWidth;
	int videoHeight;

	BrandModelOutputConfigDTO({
		this.isSelect = false,
		this.videoWidth = 0,
		this.videoHeight = 0,
	});

	factory BrandModelOutputConfigDTO.fromJson(Map<String, dynamic> map) {
		return BrandModelOutputConfigDTO( 
			isSelect: map['IsSelect'],
			videoWidth: map['VideoWidth'],
			videoHeight: map['VideoHeight'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		map['IsSelect'] = isSelect;
		map['VideoWidth'] = videoWidth;
		map['VideoHeight'] = videoHeight;
		return map;
	}
}

class SyncBrandModelOutputConfigRequest extends TokenRequest{
	String? brand;
	String? model;
	String? shortCode;

	SyncBrandModelOutputConfigRequest({
		this.brand,
		this.model,
		this.shortCode,
		String? token,
	}) : super(
			token: token,
		);

	factory SyncBrandModelOutputConfigRequest.fromJson(Map<String, dynamic> map) {
		return SyncBrandModelOutputConfigRequest( 
			brand: map['Brand'],
			model: map['Model'],
			shortCode: map['ShortCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(brand != null)
			map['Brand'] = brand;
		if(model != null)
			map['Model'] = model;
		if(shortCode != null)
			map['ShortCode'] = shortCode;
		return map;
	}
}

class GetBrandsRequest extends TokenRequest{

	GetBrandsRequest({
		String? token,
	}) : super(
			token: token,
		);

	factory GetBrandsRequest.fromJson(Map<String, dynamic> map) {
		return GetBrandsRequest( 
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		return map;
	}
}

class GetModelsRequest extends TokenRequest{
	String? brand;

	GetModelsRequest({
		this.brand,
		String? token,
	}) : super(
			token: token,
		);

	factory GetModelsRequest.fromJson(Map<String, dynamic> map) {
		return GetModelsRequest( 
			brand: map['Brand'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(brand != null)
			map['Brand'] = brand;
		return map;
	}
}

class ModifyEmergencyDeviceCodeRequest extends TokenRequest{
	String? emergencyDeviceCode;

	ModifyEmergencyDeviceCodeRequest({
		this.emergencyDeviceCode,
		String? token,
	}) : super(
			token: token,
		);

	factory ModifyEmergencyDeviceCodeRequest.fromJson(Map<String, dynamic> map) {
		return ModifyEmergencyDeviceCodeRequest( 
			emergencyDeviceCode: map['EmergencyDeviceCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(emergencyDeviceCode != null)
			map['EmergencyDeviceCode'] = emergencyDeviceCode;
		return map;
	}
}


