import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'lab.m.dart';

import 'liveConsultation.m.dart';


class LabService extends JsonRpcClientBase {
	LabService(
		String host, {
		String serviceName = "ILabService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => RecordLaboratoryFileResult.fromJson(map));
		FJsonConvert.setDecoder((map) => LabFolderDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => PageResult<RemedicalInfoDTO>.fromJson(map));
		FJsonConvert.setDecoder((map) => RemedicalInfoDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => DownloadLabFileTemplateResult.fromJson(map));
	}

	Future<RecordLaboratoryFileResult> recordLaboratoryFile(RecordLaboratoryFileRequest request) async {
		var rpcRst = await call("RecordLaboratoryFile", request);
		var result = RecordLaboratoryFileResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<List<LabFolderDTO>> getLabFolderListAsync(TokenRequest request) async {
		var rpcRst = await call("GetLabFolderListAsync", request);
		var result = (rpcRst as List).map((e)=>LabFolderDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<PageResult<RemedicalInfoDTO>> getFileListByFolderAsync(GetFileListByFolderRequest request) async {
		var rpcRst = await call("GetFileListByFolderAsync", request);
		var result = PageResult<RemedicalInfoDTO>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<DownloadLabFileTemplateResult> downloadLabFileTemplateAsync(DownloadLabFileTemplateRequest request) async {
		var rpcRst = await call("DownloadLabFileTemplateAsync", request);
		var result = DownloadLabFileTemplateResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

}

