import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'login.m.dart';

import 'liveConsultation.m.dart';


class LoginService extends JsonRpcClientBase {
	LoginService(
		String host, {
		String serviceName = "ILoginService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => LoginResult.fromJson(map));
		FJsonConvert.setDecoder((map) => StartAssociatedWithAccountResult.fromJson(map));
		FJsonConvert.setDecoder((map) => GetAssociatedAccountInfoResult.fromJson(map));
		FJsonConvert.setDecoder((map) => GetScanCodeResult.fromJson(map));
		FJsonConvert.setDecoder((map) => CheckConfirmScanStateResult.fromJson(map));
	}

	Future<LoginResult> commonLoginAsync(CommonLoginRequest request) async {
		var rpcRst = await call("CommonLoginAsync", request);
		var result = LoginResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<int> checkLoginTypeAsync(CheckLoginTypeRequest request) async {
		var rpcRst = await call("CheckLoginTypeAsync", request);
		return rpcRst;
	}

	Future<bool> commonSignUpAsync(CommonSignUpRequest request) async {
		var rpcRst = await call("CommonSignUpAsync", request);
		return rpcRst;
	}

	Future<bool> checkSMSVerificationCodeAsync(CheckSMSVerificationCodeRequest request) async {
		var rpcRst = await call("CheckSMSVerificationCodeAsync", request);
		return rpcRst;
	}

	Future<bool> sendSMSVerificationCodeAsync(SendSMSVerificationCodeRequest request) async {
		var rpcRst = await call("SendSMSVerificationCodeAsync", request);
		return rpcRst;
	}

	Future<bool> sendEmailVerificationCodeAsync(SendEmailVerificationCodeRequest request) async {
		var rpcRst = await call("SendEmailVerificationCodeAsync", request);
		return rpcRst;
	}

	Future<bool> checkEmailVerificationCodeAsync(CheckEmailVerificationCodeRequest request) async {
		var rpcRst = await call("CheckEmailVerificationCodeAsync", request);
		return rpcRst;
	}

	Future<bool> retrievePasswordByPhoneAsync(RetrievePasswordByPhoneRequest request) async {
		var rpcRst = await call("RetrievePasswordByPhoneAsync", request);
		return rpcRst;
	}

	Future<bool> retrievePasswordByEmailAsync(RetrievePasswordByEmailRequest request) async {
		var rpcRst = await call("RetrievePasswordByEmailAsync", request);
		return rpcRst;
	}

	Future<bool> verifyAccountAsync(VerifyAccountRequest request) async {
		var rpcRst = await call("VerifyAccountAsync", request);
		return rpcRst;
	}

	Future<bool> modifyPasswordAsync(ModifyPasswordRequest request) async {
		var rpcRst = await call("ModifyPasswordAsync", request);
		return rpcRst;
	}

	Future<String> generateNewPasswordAsync(GenerateNewPasswordRequest request) async {
		var rpcRst = await call("GenerateNewPasswordAsync", request);
		return rpcRst;
	}

	Future<bool> loginOutAsync(TokenRequest request) async {
		var rpcRst = await call("LoginOutAsync", request);
		return rpcRst;
	}

	Future<StartAssociatedWithAccountResult> startAssociatedWithAccountAsync(StartAssociatedWithAccountRequest request) async {
		var rpcRst = await call("StartAssociatedWithAccountAsync", request);
		var result = StartAssociatedWithAccountResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<GetAssociatedAccountInfoResult> getAssociatedAccountInfoAsync(GetAssociatedAccountInfoRequest request) async {
		var rpcRst = await call("GetAssociatedAccountInfoAsync", request);
		var result = GetAssociatedAccountInfoResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<GetScanCodeResult> getScanCodeAsync(GetScanCodeRequest request) async {
		var rpcRst = await call("GetScanCodeAsync", request);
		var result = GetScanCodeResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> confirmScanAsync(ConfirmScanRequest request) async {
		var rpcRst = await call("ConfirmScanAsync", request);
		return rpcRst;
	}

	Future<CheckConfirmScanStateResult> checkConfirmScanStateAsync(CheckConfirmScanStateRequest request) async {
		var rpcRst = await call("CheckConfirmScanStateAsync", request);
		var result = CheckConfirmScanStateResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

}

