import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'recordInfo.m.dart';

import 'liveConsultation.m.dart';
import 'patient.m.dart';


class RecordInfoService extends JsonRpcClientBase {
	RecordInfoService(
		String host, {
		String serviceName = "IRecordInfoService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => PageResult<GetRecordsPageDTO>.fromJson(map));
		FJsonConvert.setDecoder((map) => GetRecordsPageDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => QueryRecordResult.fromJson(map));
		FJsonConvert.setDecoder((map) => ProcessRecordDataResult.fromJson(map));
		FJsonConvert.setDecoder((map) => RemedicalMeasuredInfoDTO.fromJson(map));
	}

	Future<bool> createRecordInfoAsync(CreateRecordRequest request) async {
		var rpcRst = await call("CreateRecordInfoAsync", request);
		return rpcRst;
	}

	Future<PageResult<GetRecordsPageDTO>> getRecordInfoPagesAsync(GetRecordsPageRequest request) async {
		var rpcRst = await call("GetRecordInfoPagesAsync", request);
		var result = PageResult<GetRecordsPageDTO>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<QueryRecordResult> queryRecordInfoAsync(QueryRecordRequest request) async {
		var rpcRst = await call("QueryRecordInfoAsync", request);
		var result = QueryRecordResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<int> getUnReadExamNumAsync(TokenRequest request) async {
		var rpcRst = await call("GetUnReadExamNumAsync", request);
		return rpcRst;
	}

	Future<ProcessRecordDataResult> processRecordDataAsync(ProcessRecordDataRequest request) async {
		var rpcRst = await call("ProcessRecordDataAsync", request);
		var result = ProcessRecordDataResult.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> relevanceRecordAsync(RelevanceRecordRequest request) async {
		var rpcRst = await call("RelevanceRecordAsync", request);
		return rpcRst;
	}

	Future<bool> finishRecordAsync(FinishRecordRequest request) async {
		var rpcRst = await call("FinishRecordAsync", request);
		return rpcRst;
	}

	Future<bool> addRemedicalMeasuredInfoAsync(AddRemedicalMeasuredInfoRequest request) async {
		var rpcRst = await call("AddRemedicalMeasuredInfoAsync", request);
		return rpcRst;
	}

	Future<List<RemedicalMeasuredInfoDTO>> findRemedicalMeasuredInfoAsync(FindRemedicalMeasuredInfoRequest request) async {
		var rpcRst = await call("FindRemedicalMeasuredInfoAsync", request);
		var result = (rpcRst as List).map((e)=>RemedicalMeasuredInfoDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

}

