import 'liveConsultation.m.dart';
import 'patient.m.dart';

import 'package:fis_jsonrpc/utils.dart';

class PatientInfoExt {
	String? patientScanType;
	List<DataItemDTO >? content;

	PatientInfoExt({
		this.patientScanType,
		this.content,
	});

	factory PatientInfoExt.fromJson(Map<String, dynamic> map) {
		return PatientInfoExt( 
			patientScanType: map['PatientScanType'],
			content: map['Content'] != null ? (map['Content'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(patientScanType != null)
			map['PatientScanType'] = patientScanType;
		if(content != null)
			map['Content'] = content;
		return map;
	}
}

class CreateRecordRequest extends TokenRequest{
	String? patientCode;
	String? deviceCode;
	List<PatientInfoExt >? patientInfoExtList;

	CreateRecordRequest({
		this.patientCode,
		this.deviceCode,
		this.patientInfoExtList,
		String? token,
	}) : super(
			token: token,
		);

	factory CreateRecordRequest.fromJson(Map<String, dynamic> map) {
		return CreateRecordRequest( 
			patientCode: map['PatientCode'],
			deviceCode: map['DeviceCode'],
			patientInfoExtList: map['PatientInfoExtList'] != null ? (map['PatientInfoExtList'] as List).map((e)=>PatientInfoExt.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(patientCode != null)
			map['PatientCode'] = patientCode;
		if(deviceCode != null)
			map['DeviceCode'] = deviceCode;
		if(patientInfoExtList != null)
			map['PatientInfoExtList'] = patientInfoExtList;
		return map;
	}
}

enum QueryRecordStatusEnum {
	All,
	NotScanned,
	Uploaded,
	NotReport,
	Completed,
	NotCompleted,
}

enum QueryRecordCreateTypeEnum {
	All,
	Reservation,
	Normal,
}

class GetRecordsPageRequest extends PageRequest{
	String? patientCode;
	QueryRecordStatusEnum queryRecordStatus;
	QueryRecordCreateTypeEnum queryRecordCreateType;

	GetRecordsPageRequest({
		this.patientCode,
		this.queryRecordStatus = QueryRecordStatusEnum.All,
		this.queryRecordCreateType = QueryRecordCreateTypeEnum.All,
		int pageIndex = 0,
		int pageSize = 0,
		String? token,
	}) : super(
			pageIndex: pageIndex,
			pageSize: pageSize,
			token: token,
		);

	factory GetRecordsPageRequest.fromJson(Map<String, dynamic> map) {
		return GetRecordsPageRequest( 
			patientCode: map['PatientCode'],
			queryRecordStatus: QueryRecordStatusEnum.values.firstWhere((e) => e.index == map['QueryRecordStatus']),
			queryRecordCreateType: QueryRecordCreateTypeEnum.values.firstWhere((e) => e.index == map['QueryRecordCreateType']),
			pageIndex: map['PageIndex'],
			pageSize: map['PageSize'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(patientCode != null)
			map['PatientCode'] = patientCode;
		map['QueryRecordStatus'] = queryRecordStatus.index;
		map['QueryRecordCreateType'] = queryRecordCreateType.index;
		return map;
	}
}

class QueryRecordResult {
	DateTime? createTime;
	String? deptName;
	String? patientName;
	String? patientAge;
	List<DataItemDTO >? patientAgeInfo;
	int patientSex;
	String? creatorName;
	String? deviceName;
	RecordStatusEnum recordStatus;
	List<PatientInfoExt >? patientInfoExtList;
	DiagnosisStatusEnum diagnosisStatus;
	List<DiagnosisInfoDTO >? diagnosisInfos;

	QueryRecordResult({
		this.createTime,
		this.deptName,
		this.patientName,
		this.patientAge,
		this.patientAgeInfo,
		this.patientSex = 0,
		this.creatorName,
		this.deviceName,
		this.recordStatus = RecordStatusEnum.NotScanned,
		this.patientInfoExtList,
		this.diagnosisStatus = DiagnosisStatusEnum.NotRequired,
		this.diagnosisInfos,
	});

	factory QueryRecordResult.fromJson(Map<String, dynamic> map) {
		return QueryRecordResult( 
			createTime: map['CreateTime'] != null ? DateTime.parse(map['CreateTime']) : null,
			deptName: map['DeptName'],
			patientName: map['PatientName'],
			patientAge: map['PatientAge'],
			patientAgeInfo: map['PatientAgeInfo'] != null ? (map['PatientAgeInfo'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			patientSex: map['PatientSex'],
			creatorName: map['CreatorName'],
			deviceName: map['DeviceName'],
			recordStatus: RecordStatusEnum.values.firstWhere((e) => e.index == map['RecordStatus']),
			patientInfoExtList: map['PatientInfoExtList'] != null ? (map['PatientInfoExtList'] as List).map((e)=>PatientInfoExt.fromJson(e as Map<String,dynamic>)).toList() : null,
			diagnosisStatus: DiagnosisStatusEnum.values.firstWhere((e) => e.index == map['DiagnosisStatus']),
			diagnosisInfos: map['DiagnosisInfos'] != null ? (map['DiagnosisInfos'] as List).map((e)=>DiagnosisInfoDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(createTime != null)
			map['CreateTime'] = JsonRpcUtils.dateFormat(createTime!);
		if(deptName != null)
			map['DeptName'] = deptName;
		if(patientName != null)
			map['PatientName'] = patientName;
		if(patientAge != null)
			map['PatientAge'] = patientAge;
		if(patientAgeInfo != null)
			map['PatientAgeInfo'] = patientAgeInfo;
		map['PatientSex'] = patientSex;
		if(creatorName != null)
			map['CreatorName'] = creatorName;
		if(deviceName != null)
			map['DeviceName'] = deviceName;
		map['RecordStatus'] = recordStatus.index;
		if(patientInfoExtList != null)
			map['PatientInfoExtList'] = patientInfoExtList;
		map['DiagnosisStatus'] = diagnosisStatus.index;
		if(diagnosisInfos != null)
			map['DiagnosisInfos'] = diagnosisInfos;
		return map;
	}
}

class QueryRecordRequest extends TokenRequest{
	String? recordCode;

	QueryRecordRequest({
		this.recordCode,
		String? token,
	}) : super(
			token: token,
		);

	factory QueryRecordRequest.fromJson(Map<String, dynamic> map) {
		return QueryRecordRequest( 
			recordCode: map['RecordCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		return map;
	}
}

class ProcessRecordDataResult {
	List<DataItemDTO >? content;

	ProcessRecordDataResult({
		this.content,
	});

	factory ProcessRecordDataResult.fromJson(Map<String, dynamic> map) {
		return ProcessRecordDataResult( 
			content: map['Content'] != null ? (map['Content'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(content != null)
			map['Content'] = content;
		return map;
	}
}

enum AnimalSpeciesEnum {
	Bovine,
	Canidae,
	Equidae,
	Felidae,
	Caprinae,
	Suidae,
}

class ProcessRecordDataRequest extends TokenRequest{
	String? methodName;
	List<DataItemDTO >? content;
	OrganizationPatientTypeEnum patientType;
	AnimalSpeciesEnum speciesEnum;

	ProcessRecordDataRequest({
		this.methodName,
		this.content,
		this.patientType = OrganizationPatientTypeEnum.Person,
		this.speciesEnum = AnimalSpeciesEnum.Bovine,
		String? token,
	}) : super(
			token: token,
		);

	factory ProcessRecordDataRequest.fromJson(Map<String, dynamic> map) {
		return ProcessRecordDataRequest( 
			methodName: map['MethodName'],
			content: map['Content'] != null ? (map['Content'] as List).map((e)=>DataItemDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			patientType: OrganizationPatientTypeEnum.values.firstWhere((e) => e.index == map['PatientType']),
			speciesEnum: AnimalSpeciesEnum.values.firstWhere((e) => e.index == map['SpeciesEnum']),
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(methodName != null)
			map['MethodName'] = methodName;
		if(content != null)
			map['Content'] = content;
		map['PatientType'] = patientType.index;
		map['SpeciesEnum'] = speciesEnum.index;
		return map;
	}
}

class RelevanceRecordRequest extends TokenRequest{
	String? examCode;
	String? rservationCode;

	RelevanceRecordRequest({
		this.examCode,
		this.rservationCode,
		String? token,
	}) : super(
			token: token,
		);

	factory RelevanceRecordRequest.fromJson(Map<String, dynamic> map) {
		return RelevanceRecordRequest( 
			examCode: map['ExamCode'],
			rservationCode: map['RservationCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(examCode != null)
			map['ExamCode'] = examCode;
		if(rservationCode != null)
			map['RservationCode'] = rservationCode;
		return map;
	}
}

class FinishRecordRequest extends TokenRequest{
	String? recordCode;

	FinishRecordRequest({
		this.recordCode,
		String? token,
	}) : super(
			token: token,
		);

	factory FinishRecordRequest.fromJson(Map<String, dynamic> map) {
		return FinishRecordRequest( 
			recordCode: map['RecordCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		return map;
	}
}

class AddRemedicalMeasuredInfoDTO {
	String? remedicalCode;
	int frameIndex;
	String? measuredFileToken;
	String? previewFileToken;
	String? measuredData;

	AddRemedicalMeasuredInfoDTO({
		this.remedicalCode,
		this.frameIndex = 0,
		this.measuredFileToken,
		this.previewFileToken,
		this.measuredData,
	});

	factory AddRemedicalMeasuredInfoDTO.fromJson(Map<String, dynamic> map) {
		return AddRemedicalMeasuredInfoDTO( 
			remedicalCode: map['RemedicalCode'],
			frameIndex: map['FrameIndex'],
			measuredFileToken: map['MeasuredFileToken'],
			previewFileToken: map['PreviewFileToken'],
			measuredData: map['MeasuredData'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = Map<String, dynamic>();
		if(remedicalCode != null)
			map['RemedicalCode'] = remedicalCode;
		map['FrameIndex'] = frameIndex;
		if(measuredFileToken != null)
			map['MeasuredFileToken'] = measuredFileToken;
		if(previewFileToken != null)
			map['PreviewFileToken'] = previewFileToken;
		if(measuredData != null)
			map['MeasuredData'] = measuredData;
		return map;
	}
}

class AddRemedicalMeasuredInfoRequest extends TokenRequest{
	BusinessTypeEnum businessType;
	String? recordCode;
	List<AddRemedicalMeasuredInfoDTO >? remedicalMeasuredInfos;

	AddRemedicalMeasuredInfoRequest({
		this.businessType = BusinessTypeEnum.RemoteDiagnosis,
		this.recordCode,
		this.remedicalMeasuredInfos,
		String? token,
	}) : super(
			token: token,
		);

	factory AddRemedicalMeasuredInfoRequest.fromJson(Map<String, dynamic> map) {
		return AddRemedicalMeasuredInfoRequest( 
			businessType: BusinessTypeEnum.values.firstWhere((e) => e.index == map['BusinessType']),
			recordCode: map['RecordCode'],
			remedicalMeasuredInfos: map['RemedicalMeasuredInfos'] != null ? (map['RemedicalMeasuredInfos'] as List).map((e)=>AddRemedicalMeasuredInfoDTO.fromJson(e as Map<String,dynamic>)).toList() : null,
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['BusinessType'] = businessType.index;
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		if(remedicalMeasuredInfos != null)
			map['RemedicalMeasuredInfos'] = remedicalMeasuredInfos;
		return map;
	}
}

class FindRemedicalMeasuredInfoRequest extends TokenRequest{
	BusinessTypeEnum businessType;
	String? recordCode;

	FindRemedicalMeasuredInfoRequest({
		this.businessType = BusinessTypeEnum.RemoteDiagnosis,
		this.recordCode,
		String? token,
	}) : super(
			token: token,
		);

	factory FindRemedicalMeasuredInfoRequest.fromJson(Map<String, dynamic> map) {
		return FindRemedicalMeasuredInfoRequest( 
			businessType: BusinessTypeEnum.values.firstWhere((e) => e.index == map['BusinessType']),
			recordCode: map['RecordCode'],
			token: map['Token'],
		);
	}

	Map<String, dynamic> toJson() {
		final map = super.toJson();
		map['BusinessType'] = businessType.index;
		if(recordCode != null)
			map['RecordCode'] = recordCode;
		return map;
	}
}


