import 'dart:core';

import 'package:fis_jsonrpc/client_base.dart';
import 'package:fis_common/json_convert.dart';

import 'user.m.dart';

import 'liveConsultation.m.dart';


class UserService extends JsonRpcClientBase {
	UserService(
		String host, {
		String serviceName = "IUserService",
		Map<String, String>? headers,
		int? timeout,
	}) : super(
						host,
						serviceName,
						headers: headers,
						timeout: timeout,
				) {
		/// 注册响应实体反序列化处理器
		FJsonConvert.setDecoder((map) => UserDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => UserExtendDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => PageResult<ShareDeviceUserDTO>.fromJson(map));
		FJsonConvert.setDecoder((map) => ShareDeviceUserDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => UserFeatureInfoResult.fromJson(map));
		FJsonConvert.setDecoder((map) => UserInfoByCodeDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => ClientScheduleDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => PageResult<MessageExtendInfoDTO>.fromJson(map));
		FJsonConvert.setDecoder((map) => MessageExtendInfoDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => PageResult<AnnouncementExtendInfoDTO>.fromJson(map));
		FJsonConvert.setDecoder((map) => AnnouncementExtendInfoDTO.fromJson(map));
		FJsonConvert.setDecoder((map) => NoReadMessagesDTO.fromJson(map));
	}

	Future<bool> heartRateAsync(TokenRequest request) async {
		var rpcRst = await call("HeartRateAsync", request);
		return rpcRst;
	}

	Future<UserDTO> getUserInfoAsync(GetUserInfoRequest request) async {
		var rpcRst = await call("GetUserInfoAsync", request);
		var result = UserDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> alterUserInfoAsync(AlterUserInfoRequest request) async {
		var rpcRst = await call("AlterUserInfoAsync", request);
		return rpcRst;
	}

	Future<List<UserExtendDTO>> getUserListAsync(GetUserListRequest request) async {
		var rpcRst = await call("GetUserListAsync", request);
		var result = (rpcRst as List).map((e)=>UserExtendDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<bool> removeUsersFromOrganizationAsync(RemoveUsersFromOrganizationRequest request) async {
		var rpcRst = await call("RemoveUsersFromOrganizationAsync", request);
		return rpcRst;
	}

	Future<bool> setUserOrganizationInfoAsync(SetUserOrganizationInfoRequest request) async {
		var rpcRst = await call("SetUserOrganizationInfoAsync", request);
		return rpcRst;
	}

	Future<bool> alterPersonInfoAsync(AlterPersonInfoRequest request) async {
		var rpcRst = await call("AlterPersonInfoAsync", request);
		return rpcRst;
	}

	Future<PageResult<ShareDeviceUserDTO>> getShareDeviceUsersAsync(GetShareDeviceUsersPageRequest request) async {
		var rpcRst = await call("GetShareDeviceUsersAsync", request);
		var result = PageResult<ShareDeviceUserDTO>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<List<UserFeatureInfoResult>> getUserFeatureListAsync(TokenRequest request) async {
		var rpcRst = await call("GetUserFeatureListAsync", request);
		var result = (rpcRst as List).map((e)=>UserFeatureInfoResult.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<UserInfoByCodeDTO> getUserByCodeAsync(GetUserByCodeRequest request) async {
		var rpcRst = await call("GetUserByCodeAsync", request);
		var result = UserInfoByCodeDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> isDefaultPasswordAsync(TokenRequest request) async {
		var rpcRst = await call("IsDefaultPasswordAsync", request);
		return rpcRst;
	}

	Future<bool> setCommonSettingsAsync(CommonSettingsRequest request) async {
		var rpcRst = await call("SetCommonSettingsAsync", request);
		return rpcRst;
	}

	Future<bool> refreshStaticticRecordsAsync(RefreshStaticticRecordsRequest request) async {
		var rpcRst = await call("RefreshStaticticRecordsAsync", request);
		return rpcRst;
	}

	Future<List<ClientScheduleDTO>> findSchedulesAsync(FindSchedulesRequest request) async {
		var rpcRst = await call("FindSchedulesAsync", request);
		var result = (rpcRst as List).map((e)=>ClientScheduleDTO.fromJson(e as Map<String, dynamic>)).toList();
		return result;
	}

	Future<PageResult<MessageExtendInfoDTO>> getMessagesPageAsync(QueryMessageListRequest request) async {
		var rpcRst = await call("GetMessagesPageAsync", request);
		var result = PageResult<MessageExtendInfoDTO>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> setMessageDeliveryAsync(SetMessageDeliveryRequest request) async {
		var rpcRst = await call("SetMessageDeliveryAsync", request);
		return rpcRst;
	}

	Future<bool> batchSetMessageStatusAsync(SetMessageInfoReqeust request) async {
		var rpcRst = await call("BatchSetMessageStatusAsync", request);
		return rpcRst;
	}

	Future<int> getActiveUsersCount() async {
		var rpcRst = await call("GetActiveUsersCount", );
		return rpcRst;
	}

	Future<PageResult<AnnouncementExtendInfoDTO>> queryAnnouncementsByPageAsync(QueryAnnouncementListRequest request) async {
		var rpcRst = await call("QueryAnnouncementsByPageAsync", request);
		var result = PageResult<AnnouncementExtendInfoDTO>.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<AnnouncementExtendInfoDTO> getAnnouncementInfoAsync(GetAnnouncementRequest request) async {
		var rpcRst = await call("GetAnnouncementInfoAsync", request);
		var result = AnnouncementExtendInfoDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<NoReadMessagesDTO> getNoReadMessagesAsync(GetNoReadMessagesRequest request) async {
		var rpcRst = await call("GetNoReadMessagesAsync", request);
		var result = NoReadMessagesDTO.fromJson(rpcRst as Map<String, dynamic>);
		return result;
	}

	Future<bool> removeUserSingleTokenAsync(RemoveUserSingleTokenRequest request) async {
		var rpcRst = await call("RemoveUserSingleTokenAsync", request);
		return rpcRst;
	}

}

